#
# $Id: Navigation.pm,v 1.3 2004/11/09 17:06:39 rodney Exp $
#
# Copyright (C) Rodney Volz 2000, 2001
#
package PHTML::Navigation;
use strict;
use warnings;


sub new {
    my ($this,$tree) = @_;
    my $class = ref ($this) || $this; # class or object
    my $self = {};

    bless($self, $class);

    $self->{tree} = $tree;

    return($self);
}





#
# init($q) is called for each http request.
#
# $q    http Request
#
sub init {
  my ($self,$q) = @_;

  
}




sub render
{
    my ($self,$c) = @_;

    my $tree = $self->{tree};

    my $out;
    my $depth = 0;


#    $out .= "not yet<BR>\n";


    # Default-Selects ausfuehren, d.h. $where komplettieren
    my $where = $c->{where};



    $out .= "<TABLE>\n";

    $out .= $self->render_lines({ context => $c->{context},
				  tree => $tree,
				  where => $c->{where},
				  debug => $c->{debug},
			     });
    $out .= "</TABLE>\n";


    return($out);
}



# Schleifen-Implementierung, nicht rekursiv
sub render_lines
{
    my ($self,$c) = @_;

    my $out;

    my $tree = $c->{tree};
    my $where = $c->{where};

    my @where = split('/',$where,0);
    my $where_now;

    my @open;

    # Falls ein Parameter {open => "..." } uebergeben wurde,
    # diesen verwenden (sonst, "open"'s aus @$tree).
    @open = split('/',$c->{open},0) if ($c->{open});

    my $cnt = 0;
    for (;;)
    {

	if ($c->{debug})
	{
	    $out .= "<TR><TD COLSPAN=3>where=$where, where[$cnt] = $where[$cnt], cnt = $cnt";
	    $out .= " open=" . join("/",@open) . "\n"; 
	    $out .= "</TD></TR>\n"; # DEBUG
	}
	
	
	my $subtree;
	$out .= "<TR CLASS=nav>\n";
	foreach my $e (@$tree)
	{
	    # Platzhalter-Feld?
	    if (!defined($e->{name}))
	    {
		$out .= "<TD CLASS=navempty></TD>\n";
		next;
	    }

	    # Open-Default gefunden?
	    if ($e->{open} && $#open == -1)
	    {
		@open = split('/',$e->{open},0);
	    }

	    undef(@open) if ($where[$cnt] && $where[$cnt] ne $open[$cnt]);

	    my $url = $c->{URLprepend};
#	$url .= $e->{url}; # DEBUG
	    if ($url =~ /\?/)
	    {
		$url .= "&nav_where=";
	    }
	    else
	    {
		$url .= "?nav_where=";
	    }
#	    $url .= "$w/" if ($w);

	    my @w = (@open ? @open : @where);
	    $url .= join("/",(splice(@w,0,$cnt),$e->{name}));
	    
	    my $td_class = "nav";
	    

	    # Ist dieser Menuepunkt a) im @open-Pfad u. b) mit Sub-Index?
	    if ($e->{name} eq $open[$cnt] && $e->{index})
	    {
		$subtree = $e->{index};
	    }


	    # Ist dieser Menuepunkt selektiert?
	    if ($e->{name} eq $where[$cnt])
	    {
		$td_class = "navselected";
		
		if ($e->{index})
		{
		    $subtree = $e->{index};
		}
	    }
	    
	    $out .= "<TD CLASS=$td_class ALIGN=CENTER>\n";
	    $out .= "<A HREF=\"$url\">";
	    $out .= "$e->{name}</A></TD>\n";
	}
	$out .= "</TR>\n";


	# Wurde ein Subtree gefunden? Wenn ja, anzeigen,
	# sonst Ende.
	last unless ($subtree);
	$tree = $subtree;


	last if ($cnt++ > 6); # DEBUG
    }



    return($out);
}





# Rekursive Implementierung
sub render_line
{
    my ($self,$c) = @_;

    my $out;

    my $tree = $c->{tree};
    my $depth = $c->{depth} || 0;

    my $sub_index;
    my $default;

    my $where = $c->{where};

    my @sel = split('/',$where,0);



    # Wo sind wir derzeit?
    my @w;
    for (my $n = 0; $n < $depth; $n++)
    {
	push(@w,$sel[$n]);
    }
    my $w = join("/",@w);

    if ($c->{debug})
    {
	$out .= "<TR><TD COLSPAN=3>$depth, where=$where, w=$w</TD></TR>\n"; # DEBUG
    }

    $out .= "<TR CLASS=nav>\n";
    foreach my $e (@$tree)
    {
	# Platzhalter-Feld?
	if (!defined($e->{name}))
	{
	    $out .= "<TD CLASS=navempty></TD>\n";
	    next;
	}


	my $url = $c->{URLprepend};
#	$url .= $e->{url}; # DEBUG
	if ($url =~ /\?/)
	{
	    $url .= "&nav_where=";
	}
	else
	{
	    $url .= "?navi_where=";
	}
	$url .= "$w/" if ($w);
	$url .= "$e->{name}";

	my $td_class = "nav";

	# Ist dieser Menuepunkt selektiert?
	if ($e->{name} eq $sel[$depth])
	{
	    $td_class = "navselected";
	    
	    if ($e->{index})
	    {
		$sub_index = $e->{index};
	    }
	}
	
	$out .= "<TD CLASS=$td_class ALIGN=CENTER>\n";
	$out .= "<A HREF=\"$url\">";
	$out .= "$e->{name}</A></TD>\n";
    }
    $out .= "</TR>\n";

    if ($sub_index)
    {
	$out .= $self->render_line({ context => $c->{context},
				     tree => $sub_index,
				     depth => $depth + 1,
				     where => $c->{where},
				     debug => $c->{debug},
                                   });
    }

    return($out);
}







# ----------------------------------------------------------------------


sub http_enc
{
    my ($this,$s) = @_;
    my $ret;


    my @s = unpack("C*",$s);


#    $ret = join(",",@s);

    for (my $n = 0; $n <= $#s; $n++)
    {
        if ($s[$n] < 50 || $s[$n] > 120)
        {
            $ret .= sprintf("%%%02x",$s[$n]);
#            $ret .= "&#" . sprintf("%d",$s[$n]);
            next;
        }

        $ret .= pack("C",$s[$n]);
    }

    return($ret);
}


sub html_enc
{
    my ($this,$s) = @_;
    my $ret;


    my @s = unpack("C*",$s);


#    $ret = join(",",@s);

    for (my $n = 0; $n <= $#s; $n++)
    {
	my $c = $s[$n];

        if ($c < 0x41 || $c > 0x7a)
        {
#            $ret .= sprintf("%%%02x",$c);
            $ret .= "&#" . sprintf("%d",$c). ";";
            next;
        }

        $ret .= pack("C",$s[$n]);
    }

    return($ret);
}

1;

=cut

 Konzept, Draft
 ==============

 Datenstruktur
 -------------
 
 o Es gibt einen Baum (Pointer to Array of Hashes), in dem
   die Navigation abgelegt ist.
 o Jedes Element ist ein Hashref. Wenn ein Element eine
   Sub-Navigation hat, wird sie in { index => [ { ... }, ] } abgelegt.


 Rendering
 ---------

 o Der Parameter $where sagt, welcher Punkt in der Navigation
   derzeit selektiert ist. Er enthaelt durch Slashes ("/") getrennt
   die Namen der Navigationselemente. Beispiel:

   $where eq "accounting/buchungen"

 o Der Parameter $open sagt, welcher Punkt der Navigation
   angezeigt wird ("aufgeklappt"), Beispiel:

   $open eq "accounting/buchungen/korrent"

 o $open muss also mit $where beginnen. D.h.: Ein Unterpunkt
   einer nicht selektierten Navigation kann nicht ausgeklappt
   werden. (Sonst waere der aktuell selektierte Punkt
   nicht sichtbar).

 o Wenn ein Punkt selektiert ist u. einen Sub-Index hat,
   wird dieser aufgeklappt.

